"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.OpenId4VpHolderService = void 0;
const core_1 = require("@credo-ts/core");
const openid4vp_1 = require("@openid4vc/openid4vp");
const callbacks_1 = require("../shared/callbacks");
let OpenId4VpHolderService = class OpenId4VpHolderService {
    constructor(presentationExchangeService, dcqlService) {
        this.presentationExchangeService = presentationExchangeService;
        this.dcqlService = dcqlService;
    }
    getOpenid4vpClient(agentContext, options) {
        const callbacks = (0, callbacks_1.getOid4vcCallbacks)(agentContext, {
            trustedCertificates: options?.trustedCertificates,
            isVerifyOpenId4VpAuthorizationRequest: options?.isVerifyOpenId4VpAuthorizationRequest,
        });
        return new openid4vp_1.Openid4vpClient({ callbacks });
    }
    async handlePresentationExchangeRequest(agentContext, _presentationDefinition, transactionData) {
        const presentationDefinition = _presentationDefinition;
        this.presentationExchangeService.validatePresentationDefinition(presentationDefinition);
        const presentationExchange = {
            definition: presentationDefinition,
            credentialsForRequest: await this.presentationExchangeService.getCredentialsForRequest(agentContext, presentationDefinition),
        };
        const availableCredentialIds = presentationExchange.credentialsForRequest.requirements.flatMap((requirement) => requirement.submissionEntry.map((entry) => entry.inputDescriptorId));
        // for each transaction data entry, get all credentials that can be used to sign the respective transaction
        const matchedTransactionData = transactionData?.map((entry) => ({
            entry,
            matchedCredentialIds: entry.transactionData.credential_ids.filter((credentialId) => availableCredentialIds.includes(credentialId)),
        }));
        return { pex: presentationExchange, matchedTransactionData };
    }
    async handleDcqlRequest(agentContext, dcql, transactionData) {
        const dcqlQuery = this.dcqlService.validateDcqlQuery(dcql);
        const dcqlQueryResult = await this.dcqlService.getCredentialsForRequest(agentContext, dcqlQuery);
        // for each transaction data entry, get all credentials that can fore used to sign the respective transaction
        const matchedTransactionData = transactionData?.map((entry) => ({
            entry,
            matchedCredentialIds: entry.transactionData.credential_ids.filter((credentialId) => dcqlQueryResult.credential_matches[credentialId].success),
        }));
        return { dcql: { queryResult: dcqlQueryResult }, matchedTransactionData };
    }
    async resolveAuthorizationRequest(agentContext, 
    /**
     * Can be:
     * - JWT
     * - URI containing request or request_uri param
     * - Request payload
     */
    authorizationRequest, options) {
        const openid4vpClient = this.getOpenid4vpClient(agentContext, {
            trustedCertificates: options?.trustedCertificates,
            isVerifyOpenId4VpAuthorizationRequest: true,
        });
        const { params } = openid4vpClient.parseOpenid4vpAuthorizationRequest({ authorizationRequest });
        const verifiedAuthorizationRequest = await openid4vpClient.resolveOpenId4vpAuthorizationRequest({
            authorizationRequestPayload: params,
            origin: options?.origin,
        });
        const { client, pex, transactionData, dcql } = verifiedAuthorizationRequest;
        if (client.scheme !== 'x509_san_dns' && client.scheme !== 'did' && client.scheme !== 'web-origin') {
            throw new core_1.CredoError(`Client scheme '${client.scheme}' is not supported`);
        }
        const pexResult = pex?.presentation_definition
            ? await this.handlePresentationExchangeRequest(agentContext, pex.presentation_definition, transactionData)
            : undefined;
        const dcqlResult = dcql?.query ? await this.handleDcqlRequest(agentContext, dcql.query, transactionData) : undefined;
        agentContext.config.logger.debug('verified Authorization Request');
        agentContext.config.logger.debug(`request '${authorizationRequest}'`);
        return {
            authorizationRequestPayload: verifiedAuthorizationRequest.authorizationRequestPayload,
            transactionData: pexResult?.matchedTransactionData ?? dcqlResult?.matchedTransactionData,
            presentationExchange: pexResult?.pex,
            dcql: dcqlResult?.dcql,
            origin: options?.origin,
            signedAuthorizationRequest: verifiedAuthorizationRequest.jar
                ? {
                    signer: verifiedAuthorizationRequest.jar?.signer,
                    payload: verifiedAuthorizationRequest.jar.jwt.payload,
                    header: verifiedAuthorizationRequest.jar.jwt.header,
                }
                : undefined,
        };
    }
    extendCredentialsWithTransactionDataHashes(
    // Either PEX or DCQL
    selectedCredentials, transactionData, selectedTransactionDataCredentials) {
        // TODO: it would make sense for oid4vc to also handle this validation logic, but it would require
        // knowledge of PEX / DCQL...
        if (!transactionData && !selectedTransactionDataCredentials)
            return selectedCredentials;
        if (!selectedTransactionDataCredentials) {
            throw new core_1.CredoError('Autohrization request contains transaction data entries, but no credential ids to sign transaction data hashes provided in acceptAuthorizationRequest method.');
        }
        if (!transactionData) {
            throw new core_1.CredoError('Autohrization request doe not contains transaction data entries, but credentail ids were provided to sign transaction data hashes in acceptAuthorizationRequest method.');
        }
        if (transactionData.length !== selectedTransactionDataCredentials.length) {
            throw new core_1.CredoError('Credential ids to sign transaction data hashes provided in acceptAuthorizationRequest method, but the length does not match the number of transaction data entries from the authorization request.');
        }
        const credentialsToTransactionData = {};
        for (const transactionDataIndex in transactionData) {
            const transactionDataEntry = transactionData[transactionDataIndex];
            const { credentialId } = selectedTransactionDataCredentials[transactionDataIndex];
            if (!transactionDataEntry.transactionData.credential_ids.includes(credentialId)) {
                throw new core_1.CredoError(`Credential id '${credentialId}' selected to sign transaction data with index '${transactionDataIndex}' is not present in allowed credential ids for transaction. Allowed credential ids are ${transactionDataEntry.transactionData.credential_ids.join(', ')}`);
            }
            if (!selectedCredentials[credentialId]) {
                throw new core_1.CredoError(`Credential id '${credentialId}' selected to sign transaction data with index '${transactionDataIndex}', but credential is not included in the credentials for the presentation.`);
            }
            // NOTE: in the next releaes of DCQL this will also be an array, so this code can soon be simplified
            const credentialsForId = Array.isArray(selectedCredentials[credentialId])
                ? selectedCredentials[credentialId]
                : [selectedCredentials[credentialId]];
            const unsupportedFormats = credentialsForId
                .filter((c) => c.claimFormat !== core_1.ClaimFormat.SdJwtVc)
                .map((c) => c.claimFormat);
            if (unsupportedFormats.length > 0) {
                throw new core_1.CredoError(`Credential id '${credentialId}' selected to sign transaction data with index '${transactionDataIndex}' unsupported format(s) ${unsupportedFormats.join(', ')}. Only '${core_1.ClaimFormat.SdJwtVc}' is supported for transaction data signing in Credo at the moment.`);
            }
            if (!credentialsToTransactionData[credentialId]) {
                credentialsToTransactionData[credentialId] = [];
            }
            credentialsToTransactionData[credentialId].push(transactionDataEntry);
        }
        const updatedCredentials = {
            ...selectedCredentials,
        };
        for (const [credentialId, entries] of Object.entries(credentialsToTransactionData)) {
            const allowedHashAlgs = entries.reduce((allowedHashValues, entry) => (entry.transactionData.transaction_data_hashes_alg ?? ['sha-256']).filter((value) => !allowedHashValues || allowedHashValues.includes(value)), undefined);
            if (!allowedHashAlgs || allowedHashAlgs.length === 0) {
                throw new core_1.CredoError(`Unable to determine hash alg for credential with id '${credentialId}' and transaction data indexes ${entries.map((e) => e.transactionDataIndex).join(' ')}, no common 'transaction_data_hashes_alg' value found.`);
            }
            const supportedHashAlgs = ['sha-1', 'sha-256'];
            const supportedAllowedHashAlgs = supportedHashAlgs.filter((alg) => allowedHashAlgs.includes(alg));
            if (supportedAllowedHashAlgs.length === 0) {
                throw new core_1.CredoError(`Unable to create transaction data hash for credential with id '${credentialId}' and transaction data indexes ${entries.map((e) => e.transactionDataIndex).join(' ')}. None of the common allowed hash algorithms is supported by Credo: ${allowedHashAlgs.join(', ')}. Supported hash algs are ${supportedHashAlgs.join(', ')}.`);
            }
            // Not required, but we include it by default as otherwise we need to look at all entries to
            // see if any specified an alg array
            const [transactionDataHahsesAlg] = supportedAllowedHashAlgs;
            const transactionDataHashes = entries.map((entry) => core_1.TypedArrayEncoder.toBase64URL(core_1.Hasher.hash(entry.encoded, transactionDataHahsesAlg)));
            const credentialsForId = Array.isArray(updatedCredentials[credentialId])
                ? updatedCredentials[credentialId]
                : [updatedCredentials[credentialId]];
            const updatedCredentialsForId = credentialsForId.map((credential) => {
                if (credential.claimFormat !== core_1.ClaimFormat.SdJwtVc) {
                    // We already verified this above
                    throw new core_1.CredoError(`Unexpected claim format '${credential.claimFormat}' for transaction data, expected '${core_1.ClaimFormat.SdJwtVc}'`);
                }
                return {
                    ...credential,
                    additionalPayload: {
                        ...(credential.additionalPayload ?? {}),
                        transaction_data_hashes: transactionDataHashes,
                        transaction_data_hashes_alg: transactionDataHahsesAlg,
                    },
                };
            });
            // Will soon be simplified once DCQL also uses array
            updatedCredentials[credentialId] = Array.isArray(updatedCredentials[credentialId])
                ? updatedCredentialsForId
                : updatedCredentialsForId[0];
        }
        return updatedCredentials;
    }
    async acceptAuthorizationRequest(agentContext, options) {
        const { authorizationRequestPayload, presentationExchange, dcql, transactionData } = options;
        const openid4vpClient = this.getOpenid4vpClient(agentContext);
        const authorizationResponseNonce = await agentContext.wallet.generateNonce();
        const { nonce } = authorizationRequestPayload;
        const parsedClientId = (0, openid4vp_1.getOpenid4vpClientId)({
            responseMode: authorizationRequestPayload.response_mode,
            clientId: authorizationRequestPayload.client_id,
            legacyClientIdScheme: authorizationRequestPayload.client_id_scheme,
            origin: options.origin,
        });
        // If client_id_scheme was used we need to use the legacy client id.
        const clientId = parsedClientId.legacyClientId ?? parsedClientId.clientId;
        let openid4vpOptions;
        if ((0, openid4vp_1.isOpenid4vpAuthorizationRequestDcApi)(authorizationRequestPayload)) {
            if (!options.origin) {
                throw new core_1.CredoError('Missing required parameter `origin` parameter for accepting openid4vp dc api requests.');
            }
            openid4vpOptions = { type: 'openId4VpDcApi', clientId, origin: options.origin, verifierGeneratedNonce: nonce };
        }
        else {
            const responseUri = authorizationRequestPayload.response_uri ?? authorizationRequestPayload.redirect_uri;
            if (!responseUri) {
                throw new core_1.CredoError('Missing required parameter `response_uri` or `redirect_uri` in the authorization request.');
            }
            openid4vpOptions = {
                type: 'openId4Vp',
                mdocGeneratedNonce: authorizationResponseNonce,
                responseUri,
                clientId,
                verifierGeneratedNonce: nonce,
            };
        }
        let vpToken;
        let presentationSubmission = undefined;
        const parsedTransactionData = authorizationRequestPayload.transaction_data
            ? (0, openid4vp_1.parseTransactionData)({
                transactionData: authorizationRequestPayload.transaction_data,
            })
            : undefined;
        // Handle presentation exchange part
        if (authorizationRequestPayload.presentation_definition || presentationExchange) {
            if (!presentationExchange) {
                throw new core_1.CredoError('Authorization request included presentation definition. `presentationExchange` MUST be supplied to accept authorization requests.');
            }
            if (!authorizationRequestPayload.presentation_definition) {
                throw new core_1.CredoError('`presentationExchange` was supplied, but no presentation definition was found in the presentation request.');
            }
            const credentialsWithTransactionData = this.extendCredentialsWithTransactionDataHashes(presentationExchange.credentials, parsedTransactionData, transactionData);
            const { presentationSubmission: _presentationSubmission, encodedVerifiablePresentations } = await this.presentationExchangeService.createPresentation(agentContext, {
                credentialsForInputDescriptor: credentialsWithTransactionData,
                presentationDefinition: authorizationRequestPayload.presentation_definition,
                challenge: nonce,
                domain: clientId,
                presentationSubmissionLocation: core_1.DifPresentationExchangeSubmissionLocation.EXTERNAL,
                openid4vp: openid4vpOptions,
            });
            vpToken =
                encodedVerifiablePresentations.length === 1 && _presentationSubmission?.descriptor_map[0]?.path === '$'
                    ? encodedVerifiablePresentations[0]
                    : encodedVerifiablePresentations;
            presentationSubmission = _presentationSubmission;
        }
        else if (authorizationRequestPayload.dcql_query || dcql) {
            if (!authorizationRequestPayload.dcql_query) {
                throw new core_1.CredoError(`'dcql' was supplied, but no dcql request was found in the presentation request.`);
            }
            if (!dcql) {
                throw new core_1.CredoError(`Authorization request included dcql request. 'dcql' MUST be supplied to accept authorization requests.`);
            }
            const credentialsWithTransactionData = this.extendCredentialsWithTransactionDataHashes(dcql.credentials, parsedTransactionData, transactionData);
            const { encodedDcqlPresentation } = await this.dcqlService.createPresentation(agentContext, {
                credentialQueryToCredential: credentialsWithTransactionData,
                challenge: nonce,
                domain: clientId,
                openid4vp: openid4vpOptions,
            });
            vpToken = encodedDcqlPresentation;
        }
        else {
            throw new core_1.CredoError('Either pex or dcql must be provided');
        }
        const response = await openid4vpClient.createOpenid4vpAuthorizationResponse({
            authorizationRequestPayload,
            authorizationResponsePayload: {
                vp_token: vpToken,
                presentation_submission: presentationSubmission,
            },
            jarm: authorizationRequestPayload.response_mode && (0, openid4vp_1.isJarmResponseMode)(authorizationRequestPayload.response_mode)
                ? {
                    encryption: { nonce: authorizationResponseNonce },
                    serverMetadata: {
                        authorization_signing_alg_values_supported: [],
                        authorization_encryption_alg_values_supported: ['ECDH-ES'],
                        authorization_encryption_enc_values_supported: ['A128GCM', 'A256GCM', 'A128CBC-HS256'],
                    },
                }
                : undefined,
        });
        const authorizationResponsePayload = response.authorizationResponsePayload;
        const authorizationResponse = response.jarm?.responseJwt
            ? { response: response.jarm.responseJwt }
            : authorizationResponsePayload;
        // TODO: we should include more typing here that the user
        // still needs to submit the response. or as we discussed, split
        // this method up in create and submit
        if ((0, openid4vp_1.isOpenid4vpAuthorizationRequestDcApi)(authorizationRequestPayload)) {
            return {
                ok: true,
                authorizationResponse,
                authorizationResponsePayload,
            };
        }
        // TODO: parse response in openi4vp library so we can have typed error
        // as well as typed response (with redirect_uri/presentation_during_issuance_session)
        const result = await openid4vpClient.submitOpenid4vpAuthorizationResponse({
            authorizationRequestPayload,
            authorizationResponsePayload: response.authorizationResponsePayload,
            jarm: response.jarm ? { responseJwt: response.jarm.responseJwt } : undefined,
        });
        const responseText = await result.response
            .clone()
            .text()
            .catch(() => null);
        const responseJson = (await result.response
            .clone()
            .json()
            .catch(() => null));
        if (!result.response.ok) {
            return {
                ok: false,
                serverResponse: {
                    status: result.response.status,
                    body: responseJson ?? responseText,
                },
                authorizationResponse,
                authorizationResponsePayload,
            };
        }
        return {
            ok: true,
            serverResponse: {
                status: result.response.status,
                body: responseJson ?? {},
            },
            authorizationResponse,
            authorizationResponsePayload,
            redirectUri: responseJson?.redirect_uri,
            presentationDuringIssuanceSession: responseJson?.presentation_during_issuance_session,
        };
    }
};
exports.OpenId4VpHolderService = OpenId4VpHolderService;
exports.OpenId4VpHolderService = OpenId4VpHolderService = __decorate([
    (0, core_1.injectable)(),
    __metadata("design:paramtypes", [core_1.DifPresentationExchangeService,
        core_1.DcqlService])
], OpenId4VpHolderService);
//# sourceMappingURL=OpenId4vpHolderService.js.map