"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getOid4vcJwtVerifyCallback = getOid4vcJwtVerifyCallback;
exports.getOid4vcEncryptJweCallback = getOid4vcEncryptJweCallback;
exports.getOid4vcDecryptJweCallback = getOid4vcDecryptJweCallback;
exports.getOid4vcJwtSignCallback = getOid4vcJwtSignCallback;
exports.getOid4vcCallbacks = getOid4vcCallbacks;
exports.dynamicOid4vciClientAuthentication = dynamicOid4vciClientAuthentication;
const core_1 = require("@credo-ts/core");
const core_2 = require("@credo-ts/core");
const oauth2_1 = require("@openid4vc/oauth2");
const utils_1 = require("./utils");
function getOid4vcJwtVerifyCallback(agentContext, options) {
    const jwsService = agentContext.dependencyManager.resolve(core_2.JwsService);
    return async (signer, { compact, header, payload }) => {
        let trustedCertificates = options?.trustedCertificates;
        if (signer.method === 'x5c' &&
            (header.typ === 'oauth-authz-req+jwt' || options?.isAuthorizationRequestJwt) &&
            !trustedCertificates) {
            const x509Config = agentContext.dependencyManager.resolve(core_2.X509ModuleConfig);
            const certificateChain = signer.x5c?.map((cert) => core_2.X509Certificate.fromEncodedCertificate(cert));
            trustedCertificates = await x509Config.getTrustedCertificatesForVerification?.(agentContext, {
                certificateChain,
                verification: {
                    type: 'oauth2SecuredAuthorizationRequest',
                    authorizationRequest: {
                        jwt: compact,
                        payload: core_2.JwtPayload.fromJson(payload),
                    },
                },
            });
        }
        if (signer.method === 'x5c' &&
            (header.typ === 'keyattestation+jwt' || header.typ === 'key-attestation+jwt') &&
            options?.issuanceSessionId &&
            !trustedCertificates) {
            const x509Config = agentContext.dependencyManager.resolve(core_2.X509ModuleConfig);
            const certificateChain = signer.x5c?.map((cert) => core_2.X509Certificate.fromEncodedCertificate(cert));
            trustedCertificates = await x509Config.getTrustedCertificatesForVerification?.(agentContext, {
                certificateChain,
                verification: {
                    type: 'openId4VciKeyAttestation',
                    openId4VcIssuanceSessionId: options.issuanceSessionId,
                    keyAttestation: {
                        jwt: compact,
                        payload: core_2.JwtPayload.fromJson(payload),
                    },
                },
            });
        }
        if (signer.method === 'x5c' &&
            header.typ === 'oauth-client-attestation+jwt' &&
            options?.issuanceSessionId &&
            !trustedCertificates) {
            const x509Config = agentContext.dependencyManager.resolve(core_2.X509ModuleConfig);
            const certificateChain = signer.x5c?.map((cert) => core_2.X509Certificate.fromEncodedCertificate(cert));
            trustedCertificates = await x509Config.getTrustedCertificatesForVerification?.(agentContext, {
                certificateChain,
                verification: {
                    type: 'oauth2ClientAttestation',
                    openId4VcIssuanceSessionId: options.issuanceSessionId,
                    clientAttestation: {
                        jwt: compact,
                        payload: core_2.JwtPayload.fromJson(payload),
                    },
                },
            });
        }
        const alg = signer.alg;
        if (!Object.values(core_1.JwaSignatureAlgorithm).includes(alg)) {
            throw new core_2.CredoError(`Unsupported jwa signatre algorithm '${alg}'`);
        }
        const jwsSigner = signer.method === 'did'
            ? {
                method: 'did',
                didUrl: signer.didUrl,
                jwk: (0, core_2.getJwkFromKey)(await (0, utils_1.getKeyFromDid)(agentContext, signer.didUrl)),
            }
            : signer.method === 'jwk'
                ? {
                    method: 'jwk',
                    jwk: (0, core_2.getJwkFromJson)(signer.publicJwk),
                }
                : signer.method === 'x5c'
                    ? {
                        method: 'x5c',
                        x5c: signer.x5c,
                        jwk: (0, core_2.getJwkFromKey)(core_2.X509Certificate.fromEncodedCertificate(signer.x5c[0]).publicKey),
                    }
                    : undefined;
        if (!jwsSigner) {
            throw new core_2.CredoError(`Unable to verify jws with unsupported jws signer method '${signer.method}'`);
        }
        const { isValid, jwsSigners } = await jwsService.verifyJws(agentContext, {
            jws: compact,
            trustedCertificates,
            jwsSigner,
        });
        if (!isValid) {
            return { verified: false, signerJwk: undefined };
        }
        const signerJwk = jwsSigners[0].jwk.toJson();
        if (signer.method === 'did') {
            signerJwk.kid = signer.didUrl;
        }
        return { verified: true, signerJwk };
    };
}
function getOid4vcEncryptJweCallback(agentContext) {
    return async (jweEncryptor, compact) => {
        if (jweEncryptor.method !== 'jwk') {
            throw new core_2.CredoError(`Jwt encryption method '${jweEncryptor.method}' is not supported for jwt signer. Only 'jwk' is supported.`);
        }
        const jwk = (0, core_2.getJwkFromJson)(jweEncryptor.publicJwk);
        const key = jwk.key;
        if (jweEncryptor.alg !== 'ECDH-ES') {
            throw new core_2.CredoError("Only 'ECDH-ES' is supported as 'alg' value for JARM response encryption");
        }
        if (jweEncryptor.enc !== 'A256GCM' && jweEncryptor.enc !== 'A128GCM' && jweEncryptor.enc !== 'A128CBC-HS256') {
            throw new core_2.CredoError("Only 'A256GCM', 'A128GCM', and 'A128CBC-HS256' is supported as 'enc' value for JARM response encryption");
        }
        if (key.keyType !== core_2.KeyType.P256) {
            throw new core_2.CredoError(`Only '${core_2.KeyType.P256}' key type is supported for JARM response encryption`);
        }
        if (!agentContext.wallet.directEncryptCompactJweEcdhEs) {
            throw new core_2.CredoError('Cannot decrypt Jarm Response, wallet does not support directEncryptCompactJweEcdhEs. You need to upgrade your wallet implementation.');
        }
        const jwe = await agentContext.wallet.directEncryptCompactJweEcdhEs({
            data: core_2.Buffer.from(compact),
            recipientKey: key,
            header: { kid: jweEncryptor.publicJwk.kid },
            encryptionAlgorithm: jweEncryptor.enc,
            apu: jweEncryptor.apu ? core_2.TypedArrayEncoder.toBase64URL(core_2.TypedArrayEncoder.fromString(jweEncryptor.apu)) : undefined,
            apv: jweEncryptor.apv ? core_2.TypedArrayEncoder.toBase64URL(core_2.TypedArrayEncoder.fromString(jweEncryptor.apv)) : undefined,
        });
        return { encryptionJwk: jweEncryptor.publicJwk, jwe };
    };
}
function getOid4vcDecryptJweCallback(agentContext) {
    return async (jwe, options) => {
        const { header } = (0, oauth2_1.decodeJwtHeader)({ jwt: jwe });
        const kid = options?.jwk?.kid ?? header.kid;
        if (!kid) {
            throw new core_2.CredoError('Uanbel to decrypt jwe. No kid or jwk found');
        }
        const key = core_2.Key.fromFingerprint(kid);
        if (!agentContext.wallet.directDecryptCompactJweEcdhEs) {
            throw new core_2.CredoError('Cannot decrypt Jarm Response, wallet does not support directDecryptCompactJweEcdhEs');
        }
        let decryptedPayload;
        try {
            const decrypted = await agentContext.wallet.directDecryptCompactJweEcdhEs({ compactJwe: jwe, recipientKey: key });
            decryptedPayload = core_2.TypedArrayEncoder.toUtf8String(decrypted.data);
        }
        catch (_error) {
            return {
                decrypted: false,
                encryptionJwk: options?.jwk,
                payload: undefined,
                header,
            };
        }
        return {
            decrypted: true,
            decryptionJwk: (0, core_2.getJwkFromKey)(key).toJson(),
            payload: decryptedPayload,
            header,
        };
    };
}
function getOid4vcJwtSignCallback(agentContext) {
    const jwsService = agentContext.dependencyManager.resolve(core_2.JwsService);
    return async (signer, { payload, header }) => {
        if (signer.method === 'custom' || signer.method === 'federation') {
            throw new core_2.CredoError(`Jwt signer method 'custom' and 'federation' are not supported for jwt signer.`);
        }
        if (signer.method === 'x5c') {
            const leafCertificate = core_2.X509Service.getLeafCertificate(agentContext, { certificateChain: signer.x5c });
            const jws = await jwsService.createJwsCompact(agentContext, {
                protectedHeaderOptions: { ...header, alg: signer.alg, jwk: undefined },
                payload: core_2.JwtPayload.fromJson(payload),
                key: leafCertificate.publicKey,
            });
            return { jwt: jws, signerJwk: (0, core_2.getJwkFromKey)(leafCertificate.publicKey).toJson() };
        }
        const key = signer.method === 'did' ? await (0, utils_1.getKeyFromDid)(agentContext, signer.didUrl) : (0, core_2.getJwkFromJson)(signer.publicJwk).key;
        const jwk = (0, core_2.getJwkFromKey)(key);
        if (!jwk.supportsSignatureAlgorithm(signer.alg)) {
            throw new core_2.CredoError(`key type '${jwk.keyType}', does not support the JWS signature alg '${signer.alg}'`);
        }
        const jwt = await jwsService.createJwsCompact(agentContext, {
            protectedHeaderOptions: {
                ...header,
                jwk: header.jwk ? (0, core_2.getJwkFromJson)(header.jwk) : undefined,
            },
            payload: core_2.JsonEncoder.toBuffer(payload),
            key,
        });
        return { jwt, signerJwk: (0, core_2.getJwkFromKey)(key).toJson() };
    };
}
function getOid4vcCallbacks(agentContext, options) {
    return {
        hash: (data, alg) => core_2.Hasher.hash(data, alg.toLowerCase()),
        generateRandom: (length) => agentContext.wallet.getRandomValues(length),
        signJwt: getOid4vcJwtSignCallback(agentContext),
        clientAuthentication: () => {
            throw new core_2.CredoError('Did not expect client authentication to be called.');
        },
        verifyJwt: getOid4vcJwtVerifyCallback(agentContext, {
            trustedCertificates: options?.trustedCertificates,
            isAuthorizationRequestJwt: options?.isVerifyOpenId4VpAuthorizationRequest,
            issuanceSessionId: options?.issuanceSessionId,
        }),
        fetch: agentContext.config.agentDependencies.fetch,
        encryptJwe: getOid4vcEncryptJweCallback(agentContext),
        decryptJwe: getOid4vcDecryptJweCallback(agentContext),
        getX509CertificateMetadata: (certificate) => {
            const leafCertificate = core_2.X509Service.getLeafCertificate(agentContext, { certificateChain: [certificate] });
            return {
                sanDnsNames: leafCertificate.sanDnsNames,
                sanUriNames: leafCertificate.sanUriNames,
            };
        },
    };
}
/**
 * Allows us to authenticate when making requests to an external
 * authorizatin server
 */
function dynamicOid4vciClientAuthentication(agentContext, issuerRecord) {
    return (callbackOptions) => {
        const authorizationServer = issuerRecord.authorizationServerConfigs?.find((a) => a.issuer === callbackOptions.authorizationServerMetadata.issuer);
        if (!authorizationServer) {
            // No client authentication if authorization server is not configured
            agentContext.config.logger.debug(`Unknown authorization server '${callbackOptions.authorizationServerMetadata.issuer}' for issuer '${issuerRecord.issuerId}' for request to '${callbackOptions.url}'`);
            return;
        }
        if (!authorizationServer.clientAuthentication) {
            throw new core_2.CredoError(`Unable to authenticate to authorization server '${authorizationServer.issuer}' for issuer '${issuerRecord.issuerId}' for request to '${callbackOptions.url}'. Make sure to configure a 'clientId' and 'clientSecret' for the authorization server on the issuer record.`);
        }
        return (0, oauth2_1.clientAuthenticationDynamic)({
            clientId: authorizationServer.clientAuthentication.clientId,
            clientSecret: authorizationServer.clientAuthentication.clientSecret,
        })(callbackOptions);
    };
}
//# sourceMappingURL=callbacks.js.map