"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getSupportedJwaSignatureAlgorithms = getSupportedJwaSignatureAlgorithms;
exports.getKeyFromDid = getKeyFromDid;
exports.requestSignerToJwtIssuer = requestSignerToJwtIssuer;
exports.getProofTypeFromKey = getProofTypeFromKey;
exports.addSecondsToDate = addSecondsToDate;
exports.dateToSeconds = dateToSeconds;
exports.parseIfJson = parseIfJson;
const core_1 = require("@credo-ts/core");
/**
 * Returns the JWA Signature Algorithms that are supported by the wallet.
 *
 * This is an approximation based on the supported key types of the wallet.
 * This is not 100% correct as a supporting a key type does not mean you support
 * all the algorithms for that key type. However, this needs refactoring of the wallet
 * that is planned for the 0.5.0 release.
 */
function getSupportedJwaSignatureAlgorithms(agentContext) {
    const supportedKeyTypes = agentContext.wallet.supportedKeyTypes;
    // Extract the supported JWS algs based on the key types the wallet support.
    const supportedJwaSignatureAlgorithms = supportedKeyTypes
        // Map the supported key types to the supported JWK class
        .map(core_1.getJwkClassFromKeyType)
        // Filter out the undefined values
        .filter((jwkClass) => jwkClass !== undefined)
        // Extract the supported JWA signature algorithms from the JWK class
        .flatMap((jwkClass) => jwkClass.supportedSignatureAlgorithms);
    return supportedJwaSignatureAlgorithms;
}
async function getKeyFromDid(agentContext, didUrl, allowedPurposes = ['authentication']) {
    const didsApi = agentContext.dependencyManager.resolve(core_1.DidsApi);
    const didDocument = await didsApi.resolveDidDocument(didUrl);
    const verificationMethod = didDocument.dereferenceKey(didUrl, allowedPurposes);
    return (0, core_1.getKeyFromVerificationMethod)(verificationMethod);
}
async function requestSignerToJwtIssuer(agentContext, requestSigner) {
    if (requestSigner.method === 'did') {
        const key = await getKeyFromDid(agentContext, requestSigner.didUrl);
        const alg = (0, core_1.getJwkClassFromKeyType)(key.keyType)?.supportedSignatureAlgorithms[0];
        if (!alg)
            throw new core_1.CredoError(`No supported signature algorithms for key type: ${key.keyType}`);
        return {
            method: requestSigner.method,
            didUrl: requestSigner.didUrl,
            alg,
        };
    }
    if (requestSigner.method === 'x5c') {
        const leafCertificate = core_1.X509Service.getLeafCertificate(agentContext, {
            certificateChain: requestSigner.x5c,
        });
        const jwk = (0, core_1.getJwkFromKey)(leafCertificate.publicKey);
        const alg = jwk.supportedSignatureAlgorithms[0];
        if (!alg) {
            throw new core_1.CredoError(`No supported signature algorithms found key type: '${jwk.keyType}'`);
        }
        if (!requestSigner.issuer.startsWith('https://') &&
            !(requestSigner.issuer.startsWith('http://') && agentContext.config.allowInsecureHttpUrls)) {
            throw new core_1.CredoError('The X509 certificate issuer must be a HTTPS URI.');
        }
        if (!leafCertificate.sanUriNames.includes(requestSigner.issuer) &&
            !leafCertificate.sanDnsNames.includes((0, core_1.getDomainFromUrl)(requestSigner.issuer))) {
            const sanUriMessage = leafCertificate.sanUriNames.length > 0
                ? `SAN-URI names are ${leafCertificate.sanUriNames.join(', ')}`
                : 'there are no SAN-URI names';
            const sanDnsMessage = leafCertificate.sanDnsNames.length > 0
                ? `SAN-DNS names are ${leafCertificate.sanDnsNames.join(', ')}`
                : 'there are no SAN-DNS names';
            throw new Error(`The 'iss' claim in the payload does not match a 'SAN-URI' or 'SAN-DNS' name in the x5c certificate. 'iss' value is '${requestSigner.issuer}', ${sanUriMessage}, ${sanDnsMessage} (for SAN-DNS only domain has to match)`);
        }
        return {
            ...requestSigner,
            alg,
        };
    }
    if (requestSigner.method === 'jwk') {
        const alg = requestSigner.jwk.supportedSignatureAlgorithms[0];
        if (!alg) {
            throw new core_1.CredoError(`No supported signature algorithms for key type: '${requestSigner.jwk.keyType}'`);
        }
        return {
            ...requestSigner,
            publicJwk: requestSigner.jwk.toJson(),
            alg,
        };
    }
    throw new core_1.CredoError(`Unsupported jwt issuer method '${requestSigner.method}'`);
}
function getProofTypeFromKey(agentContext, key) {
    const signatureSuiteRegistry = agentContext.dependencyManager.resolve(core_1.SignatureSuiteRegistry);
    const supportedSignatureSuites = signatureSuiteRegistry.getAllByKeyType(key.keyType);
    if (supportedSignatureSuites.length === 0) {
        throw new core_1.CredoError(`Couldn't find a supported signature suite for the given key type '${key.keyType}'.`);
    }
    return supportedSignatureSuites[0].proofType;
}
function addSecondsToDate(date, seconds) {
    return new Date(date.getTime() + seconds * 1000);
}
function dateToSeconds(date) {
    return Math.floor(date.getTime() / 1000);
}
function parseIfJson(input) {
    if (typeof input !== 'string') {
        return input;
    }
    try {
        // Try to parse the string as JSON
        return JSON.parse(input);
    }
    catch (_error) {
        /* empty */
    }
    return input;
}
//# sourceMappingURL=utils.js.map